// #autoload
// #name = HalSense
// #version = 1.4
// #date = April 9, 2002
// #author = Halide
// #warrior = Halide
// #email = halide@popmail.com
// #web = http://halides.tribes-universe.com/
// #description = Alters mouse sensitivity with a sticky or toggle key.
// #credit = Crunchy - PilotMode script
// #category = Halide's Scripts

if(!isObject(HalSense))
{
	new ScriptObject(HalSense)
	{
		class = HalSense;
		version = "1.4";
		keyMap = "moveMap";
	};
}

function HalSenseSticky(%val)
{
	HalSense.set(%val);
}

function HalSenseToggle(%val)
{
	if(%val)
		HalSense.set(HalSense.toggle = !HalSense.toggle);
}

function HalSenseDlgToggle(%val)
{
	if(!%val)
		return;

	if(HalSenseDlg.isAwake())
		Canvas.popDialog(HalSenseDlg);
	else
		Canvas.pushDialog(HalSenseDlg);
}

// START package HalSense
package HalSense
{

	function HalSense::setPrefs(%this)
	{
		if(isFile("prefs/HalPrefs.cs") && !$HalPrefsLoaded)
		{
			exec("prefs/HalPrefs.cs");
			$HalPrefsLoaded = true;
		}

		if($HalPref::Sense::Version $= "")
		{
			$HalPref::Sense::Version = HalSense.version;
			$HalPref::Sense::Enabled = true;
			$HalPref::Sense::Link = true;
			$HalPref::Sense::Factor[yaw] = 2.5;
			$HalPref::Sense::Factor[pitch] = 2.5;
		}
	}

	function clientCmdSetDefaultVehicleKeys(%inVehicle)
	{
	   	parent::clientCmdSetDefaultVehicleKeys(%inVehicle);

		if(%inVehicle)
		{
			HalSense.keyMap = "passengerKeys";
			passengerKeys.copyBind(moveMap, HalSenseDlgToggle);
			passengerKeys.copyBind(moveMap, HalSenseSticky);
			passengerKeys.copyBind(moveMap, HalSenseToggle);
		}
		else
			HalSense.keyMap = "moveMap";

		if(HalSense.active)
		{
			// switch to previous sensitivity and apply factor to new keymap
			HalSense.set(false);
			HalSense.set(true);
		}
	}

	function HalSense::set(%this, %val)
	{
		HalSenseRet.setVisible(%val);

		if(%val)
		{
			if(%this.active)
				return;

			%keyMap = %this.keyMap;

			%bind = %keyMap.getBinding(yaw);
			%deviceYaw = getField(%bind, 0);
			%actionYaw = getField(%bind, 1);

			%bind = %keyMap.getBinding(pitch);
			%devicePitch = getField(%bind, 0);
			%actionPitch = getField(%bind, 1);

			%flagsYaw = %keyMap.isInverted(%deviceYaw, %actionYaw) ? "SI" : "S";
			%flagsPitch = %keyMap.isInverted(%devicePitch, %actionPitch) ? "SI" : "S";
		
			%scaleYaw = %keyMap.getScale(%deviceYaw, %actionYaw);
			%scalePitch = %keyMap.getScale(%devicePitch, %actionPitch);

			%this.prev[keyMap] = %keyMap;
			
			%this.prev[yaw] = %scaleYaw;
			%this.prev[pitch] = %scalePitch;
			
			%this.prev[flagsYaw] = %flagsYaw;
			%this.prev[flagsPitch] = %flagsPitch;
			
			%this.prev[deviceYaw] = %deviceYaw;
			%this.prev[devicePitch] = %devicePitch;

			%this.prev[actionYaw] = %actionYaw;
			%this.prev[actionPitch] = %actionPitch;

			%scaleYaw *= $HalPref::Sense::Factor[yaw];
			%scalePitch *= $HalPref::Sense::Factor[pitch];

			%keyMap.bind(%deviceYaw, %actionYaw, %flagsYaw, %scaleYaw, yaw);
			%keyMap.bind(%devicePitch, %actionPitch, %flagsPitch, %scalePitch, pitch);

			%this.active = true;
		}
		else
		{
			%this.prev[keyMap].bind(%this.prev[deviceYaw], %this.prev[actionYaw], %this.prev[flagsYaw], %this.prev[yaw], yaw);
			%this.prev[keyMap].bind(%this.prev[devicePitch], %this.prev[actionPitch], %this.prev[flagsPitch], %this.prev[pitch], pitch);

			%this.active = false;
		}
	}

	function HalSense::setup(%this)
	{
		if(%this.setup)
			return;

		%this.setPrefs();

		new GuiControl(HalSenseDlg) {
			profile = "GuiDefaultProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "0 0";
			extent = "640 480";
			minExtent = "8 8";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";

			new ShellPaneCtrl() {
				profile = "ShellDlgPaneProfile";
				horizSizing = "center";
				vertSizing = "center";
				position = "201 152";
				extent = "237 175";
				minExtent = "48 92";
				visible = "1";
				hideCursor = "0";
				bypassHideCursor = "0";
				helpTag = "0";
				text = "HalSense";
				longTextBuffer = "0";
				maxLength = "255";
				noTitleBar = "0";

				new ShellFieldCtrl() {
					profile = "ShellFieldProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "20 29";
					extent = "197 126";
					minExtent = "16 18";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					helpTag = "0";
				};
				new ShellBitmapButton() {
					profile = "ShellButtonProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "117 117";
					extent = "97 38";
					minExtent = "32 38";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					command = "Canvas.popDialog(HalSenseDlg);";
					helpTag = "0";
					text = "DONE";
					simpleStyle = "0";
				};
				new ShellToggleButton(HalSenseLink) {
					profile = "ShellRadioProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "31 122";
					extent = "81 30";
					minExtent = "26 27";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "$HalPref::Sense::Link";
					helpTag = "0";
					text = "LINK X/Y";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new GuiTextCtrl(HalSenseYawText) {
					profile = "ShellProgressBarTextProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "31 34";
					extent = "104 22";
					minExtent = "8 8";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					helpTag = "0";
					text = "X-Axis:";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new GuiTextCtrl(HalSensePitchText) {
					profile = "ShellProgressBarTextProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "31 77";
					extent = "103 22";
					minExtent = "8 8";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					helpTag = "0";
					text = "Y-Axis:";
					longTextBuffer = "0";
					maxLength = "255";
				};
				new ShellSliderCtrl(HalSenseYawSlider) {
					profile = "ShellSliderProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "31 51";
					extent = "175 24";
					minExtent = "12 24";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "value";
					altCommand = "HalSense.setFactor(yaw);";
					helpTag = "0";
					range = "0.010000 5.000000";
					ticks = "1000";
					value = "0.01";
					usePlusMinus = "1";
				};
				new ShellSliderCtrl(HalSensePitchSlider) {
					profile = "ShellSliderProfile";
					horizSizing = "right";
					vertSizing = "bottom";
					position = "31 94";
					extent = "175 24";
					minExtent = "12 24";
					visible = "1";
					hideCursor = "0";
					bypassHideCursor = "0";
					variable = "value";
					altCommand = "HalSense.setFactor(pitch);";
					helpTag = "0";
					range = "0.010000 5.000000";
					ticks = "1000";
					value = "0.01";
					usePlusMinus = "1";
				};
			};
		};

		HalSenseDlg.slider[yaw] = HalSenseYawSlider.getId();
		HalSenseDlg.slider[pitch] = HalSensePitchSlider.getId();
		HalSenseDlg.text[yaw] = HalSenseYawText.getId();
		HalSenseDlg.text[pitch] = HalSensePitchText.getId();

		HalSenseLink.setValue($HalPref::Sense::Link);
		HalSenseYawSlider.setValue($HalPref::Sense::Factor[yaw]);
		HalSensePitchSlider.setValue($HalPref::Sense::Factor[pitch]);

		new HudCrosshair(HalSenseRet) {
			profile = "GuiButtonProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = "96 96";
			extent = "64 64";
			minExtent = "8 8";
			visible = "0";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";
			fillColor = "0.250000 0.250000 0.250000 0.250000";
			frameColor = "0.000000 1.000000 0.000000 1.000000";
			opacity = "1";
			bitmap = "gui/hud_ret_halsense.png";
			autoCenter = "1";
			autoResize = "1";
			flipVertical = "0";
			flipHorizontal = "0";
		};

		retCenterHud.add(HalSenseRet);

		%this.setup = true;
	}

	function HalSenseDlg::onWake(%this)
	{
		clientCmdTogglePlayHuds(false);

		%this.text[yaw].setValue("X-Axis Factor: (" @ mFormatFloat($HalPref::Sense::Factor[yaw], "%.1f") @ "x)");
		%this.text[pitch].setValue("Y-Axis Factor: (" @ mFormatFloat($HalPref::Sense::Factor[pitch], "%.1f") @ "x)");
	}

	function HalSenseDlg::onSleep(%this)
	{
		clientCmdTogglePlayHuds(true);
	}

	function HalSense::setFactor(%this, %type)
	{
	    %factor = HalSenseDlg.slider[%type].getValue();
		$HalPref::Sense::Factor[%type] = %factor;
		%axis = (%type $= "yaw") ? "X-Axis" : "Y-Axis";
		HalSenseDlg.text[%type].setValue(%axis SPC "Factor: (" @ mFormatFloat(%factor, "%.1f") @ "x)");
		%type = (%type $= "yaw") ? "pitch" : "yaw";
		if($HalPref::Sense::Link && HalSenseDlg.slider[%type].getValue() != %factor)
			HalSenseDlg.slider[%type].setValue(%factor);
	}

	function OptionsDlg::onWake(%this)
	{
		if(!HalSense.bind)
		{
			$RemapName[$RemapCount] = "HalSense - Sticky";
			$RemapCmd[$RemapCount] = "HalSenseSticky";
			$RemapCount++;
			$RemapName[$RemapCount] = "HalSense - Toggle";
			$RemapCmd[$RemapCount] = "HalSenseToggle";
			$RemapCount++;
			$RemapName[$RemapCount] = "HalSense - Options";
			$RemapCmd[$RemapCount] = "HalSenseDlgToggle";
			$RemapCount++;

			HalSense.bind = true;
		}

		parent::onWake(%this);
	}
	
	function DispatchLaunchMode()
	{
		HalSense.setup();

		parent::DispatchLaunchMode();
	}

	function quit()
	{
		if(!$HalPrefsSaved)
		{
			export("$HalPref::*", "prefs/HalPrefs.cs", false);
			$HalPrefsSaved = true;
		}

		parent::quit();
	}

// END package HalSense
};
activatePackage(HalSense);